package main

import (
	"errors"
	"fmt"
	"math/rand"
	"maze-gen/maze"
	"os"
	"regexp"
	"strconv"
	"time"
)

// Parse argument of form "12x34" or "56*78"
func parseDimension(s string) (int, int, error) {
	re := regexp.MustCompile(`^(\d+)[x*](\d+)$`)
	matches := re.FindStringSubmatch(s)
	if len(matches) == 3 {
		width, err1 := strconv.Atoi(matches[1])
		if err1 != nil {
			return 0, 0, err1
		}
		height, err2 := strconv.Atoi(matches[2])
		return width, height, err2
	}
	return 0, 0, errors.New("invalid format")
}

func main() {
	// Init RNG
	rand.Seed(time.Now().UTC().UnixNano())

	// Parse command line arguments
	width, height := 20, 20
	if len(os.Args) > 1 {
		var err error
		width, height, err = parseDimension(os.Args[1])
		if err != nil {
			_ = fmt.Errorf("USAGE: %s 30x20", os.Args[0])
			os.Exit(1)
		}
	}
	if width > 500 || height > 500 {
		_ = fmt.Errorf("Dimensions too large!")
		os.Exit(1)
	}

	// Generate the maze
	println("Generating a maze now ...")
	start := time.Now()
	m := maze.NewMaze(width, height)
	m.GenerateRandomEntryExit()
	m.GenerateValidMaze()
	elapsed := time.Since(start)
	fmt.Printf("Maze generated, took %s\n", elapsed)
	m.ToPngImage(28, false, "maze.png")
	m.ToPngImage(28, true, "maze-solution.png")
	fmt.Printf("Wrote \"%s\" and \"%s\".\n", "maze.png", "maze-solution.png")
}
