package main

import (
	"fmt"
	"github.com/therecipe/qt/core"
	"github.com/therecipe/qt/gui"
	"github.com/therecipe/qt/uitools"
	"github.com/therecipe/qt/widgets"
	"math/rand"
	"maze-gen/maze"
	"os"
	"path/filepath"
	"strings"
	"time"
)

func NewMainForm() *widgets.QWidget {
	// The current maze
	var m *maze.Maze = nil

	// Init GUI
	file := core.NewQFile2(":/qml/mainform.ui")
	file.Open(core.QIODevice__ReadOnly)
	formWidget := uitools.NewQUiLoader(nil).Load(file, nil)
	file.Close()

	var (
		uiWidth      = widgets.NewQSpinBoxFromPointer(formWidget.FindChild("spinWidth", core.Qt__FindChildrenRecursively).Pointer())
		uiHeight     = widgets.NewQSpinBoxFromPointer(formWidget.FindChild("spinHeight", core.Qt__FindChildrenRecursively).Pointer())
		btnGenerate  = widgets.NewQPushButtonFromPointer(formWidget.FindChild("btnGenerate", core.Qt__FindChildrenRecursively).Pointer())
		btnSolve     = widgets.NewQPushButtonFromPointer(formWidget.FindChild("btnSolve", core.Qt__FindChildrenRecursively).Pointer())
		btnSaveImage = widgets.NewQPushButtonFromPointer(formWidget.FindChild("btnSaveImage", core.Qt__FindChildrenRecursively).Pointer())
		lblImage     = widgets.NewQLabelFromPointer(formWidget.FindChild("lblImage", core.Qt__FindChildrenRecursively).Pointer())
		statusbar    = widgets.NewQStatusBarFromPointer(formWidget.FindChild("statusbar", core.Qt__FindChildrenRecursively).Pointer())
	)

	drawImage := func() {
		// Repaint the current maze
		img := m.ToImage(28, m.Solved)
		image := gui.NewQImage3(img.Bounds().Max.X, img.Bounds().Max.Y, gui.QImage__Format_ARGB32)
		for y := 0; y < img.Bounds().Max.Y; y++ {
			for x := 0; x < img.Bounds().Max.X; x++ {
				c := img.RGBAAt(x, y)
				image.SetPixel2(x, y, (uint(c.R)<<16)|(uint(c.G)<<8)|(uint(c.B)<<0)|(uint(c.A)<<24))
			}
		}
		pixmap := gui.NewQPixmap2(image.Size())
		pixmap = pixmap.FromImage(image, core.Qt__AutoColor)
		lblImage.SetPixmap(pixmap)
		lblImage.Resize(lblImage.Pixmap().Size())
	}

	// Generate new maze
	btnGenerate.ConnectClicked(func(checked bool) {
		t := time.Now()
		m = maze.NewMazePtr(uiWidth.Value(), uiHeight.Value())
		m.GenerateRandomEntryExit()
		m.GenerateValidMaze()
		duration := time.Since(t)
		statusbar.ShowMessage(fmt.Sprintf("Maze %dx%d, generated in %s", uiWidth.Value(), uiHeight.Value(), duration), 0)
		btnSolve.SetEnabled(!m.Solved)
		drawImage()
	})
	// Solve current maze
	btnSolve.ConnectClicked(func(checked bool) {
		if !m.Solved {
			t := time.Now()
			m.Solve()
			duration := time.Since(t)
			statusbar.ShowMessage(fmt.Sprintf("Maze %dx%d, solved in %s", m.Width, m.Height, duration), 0)
			drawImage()
		}
		btnSolve.SetEnabled(!m.Solved)
	})
	// Save current maze as image
	btnSaveImage.ConnectClicked(func(checked bool) {
		filename := widgets.QFileDialog_GetSaveFileName(formWidget, "Save maze as image ...", ".",
			"PNG Images (*.png);;All files (*.*)", "", 0)
		fmt.Printf("filename \"%s\"\n", filename)
		if filename != "" {
			if !strings.Contains(filepath.Base(filename), ".") {
				filename += ".png"
			}
			m.ToPngImage(28, m.Solved, filename)
		}
	})
	btnGenerate.Click()

	return formWidget
}

func main() {
	// Init RNG
	rand.Seed(time.Now().UTC().UnixNano())

	// Show GUI
	widgets.NewQApplication(len(os.Args), os.Args)
	form := NewMainForm()
	form.Show()
	widgets.QApplication_Exec()
}
