package maze

import "container/heap"

func Abs(x int) int {
	if x < 0 {
		return -x
	}
	return x
}

// A priority queue entry as required by A*.
// priority = distance + |x-targetX| + |y-targetY|
type PriorityQueueEntry struct {
	x, y               int
	distance, priority int
	index              int
}

func (self *PriorityQueueEntry) ToIndex(width int) int {
	return self.x + self.y*width
}

// The priority queue, plus target field (for heuristics computation)
type PriorityQueue struct {
	data             []*PriorityQueueEntry
	targetX, targetY int
}

func NewPriorityQueue(initialEntry *PriorityQueueEntry, targetX int, targetY int) PriorityQueue {
	pq := PriorityQueue{
		data:    make([]*PriorityQueueEntry, 1),
		targetX: targetX,
		targetY: targetY,
	}
	pq.data[0] = initialEntry
	heap.Init(&pq)
	return pq
}

func (pq PriorityQueue) Len() int { return len(pq.data) }

func (pq PriorityQueue) Less(i, j int) bool {
	return pq.data[i].priority < pq.data[j].priority
}

func (pq PriorityQueue) Swap(i, j int) {
	pq.data[i], pq.data[j] = pq.data[j], pq.data[i]
	pq.data[i].index = i
	pq.data[j].index = j
}

func (pq *PriorityQueue) Push(x interface{}) {
	n := len(pq.data)
	item := x.(*PriorityQueueEntry)
	item.index = n
	// heuristics: minimal distance
	// priority = current distance + minimal distance to target
	item.priority = item.distance + Abs(item.x - pq.targetX) + Abs(item.y - pq.targetY)
	pq.data = append(pq.data, item)
}

func (pq *PriorityQueue) Pop() interface{} {
	n := len(pq.data)
	item := pq.data[n-1]
	pq.data[n-1] = nil // avoid memory leak
	item.index = -1    // for safety
	pq.data = pq.data[0 : n-1]
	return item
}

func (pq *PriorityQueue) update(item *PriorityQueueEntry) {
	item.priority = item.distance + Abs(item.x - pq.targetX) + Abs(item.y - pq.targetY)
	heap.Fix(pq, item.index)
}
